/************************************************************************
*	Modular CODEC Manager
*		Written by Horie, Seiichi
*		(c) 2002, ADKK.
*
*************************************************************************/

#ifndef INCL_MCM
#define INCL_MCM


namespace ac97{

	// codec address definition.
	// These values are passed to tAC97Write, tAC97Read, iAC97Write, iAC97Read methods
	const int  REGS_RESET 			= 0x0000;
	const int  MASTER_VOLUME		= 0x0200;
	const int  HEAD_VOLUME			= 0x0400;
	const int  MASTER_VOLUME_MONO	= 0x0600;
	const int  RESERVED_REG_2		= 0x0800;
	const int  PC_BEEP_VOLUME		= 0x0A00;
	const int  PHONE_VOLUME			= 0x0C00;
	const int  MIC_VOLUME			= 0x0E00;
	const int  LINE_IN_VOLUME		= 0x1000;
	const int  CD_VOLUME			= 0x1200;
	const int  VIDEO_VOLUME			= 0x1400;
	const int  AUX_VOLUME			= 0x1600;
	const int  PCM_OUT_VOLUME		= 0x1800;
	const int  RECORD_SELECT		= 0x1A00;
	const int  RECORD_GAIN			= 0x1C00;
	const int  RESERVED_REG_3		= 0x1E00;
	const int  GENERAL_PURPOSE		= 0x2000;
	const int  THREE_D_CONTROL_REG	= 0x2200;
	const int  RESERVED_REG_4		= 0x2400;
	const int  POWERDOWN_CTRL_STAT	= 0x2600;
	const int  EXTEND_AUDIO_ID		= 0x2800;
	const int  EXTEND_AUDIO_CTL		= 0x2A00;
	const int  SAMPLE_RATE_GENERATE_1= 0x2C00;
	const int  SAMPLE_RATE_GENERATE_0= 0x3200;
	const int  JACK_SENSE			= 0x7200;
	const int  SERIAL_CONFIGURATION	= 0x7400;
	const int  MISC_CONTROL_BITS	= 0x7600;
	const int  VENDOR_ID_1			= 0x7C00;
	const int  VENDOR_ID_2			= 0x7E00;

// utility function to construct a codec control data.
// some AC97's register has mute + lch + rch format. 
// this inline function construct that format with given parameter.
	inline int gain( int mute, int left, int right )
	{
		return ( mute ? 0x8000 : 0 ) | ( ( left & 0x01f ) <<8 ) | ( right & 0x01f );
	}


};

namespace mcm {

// error number	These numbers are passed to error() method.
	const int ERROR_CMD_OVERFLOW 		=1;
	const int ERROR_PORT_WRONG 			=2;
	const int WRONG_STATE 				=3;
	const int ERROR_NO_CRITICAL_SECTION =4;
	const int ERROR_NO_RESET 			=5;
	
// state of MCM. Only STATE_RUNNING is monitored by user code.
// Other state can be ignored.
	const int STATE_RUNNING				=31;
	const int STATE_NULL 				=0;
	const int STATE_WAIT_READY			=1;
	const int STATE_GOTO_SLOT16			=11;
	const int STATE_GOTO_SLOT16_2		=12;
	const int STATE_GOTO_SLOT16_3		=13;
	const int STATE_GOTO_SLOT16_4 		=14;
	const int STATE_SETUP_CODEC			=21;
	const int STATE_SETUP_CODEC_2		=22;
	const int STATE_SETUP_CODEC_3		=23;
	

/**************************************************************************
*
* basic type to keep the data of 1 sample. This type contains first 5 slots
* of frame. See AC97 standard.
* The first variable "dummy", is placed to keep "l" on even word align. 
* This enhance the SIMD optimization. To allow dummy variable, MCM does : 
* 1. transfere 6 slots/frame instead 5.
* 2. dummy contains last slots instead of first.
* 3. trasfer starts from tag member of first sample.
* 4. To accept abandan data, MCM reserve 1 extra sample for the end of buffer.
**************************************************************************/

	struct sample {
		short dummy, tag, address, command, l, r;	
	};

/**************************************************************************
*	
* This bit map is used to specify which slot will be transfered.
* Usually, initAndStartsSPORT() method uses this.
*	
**************************************************************************/	

	const int SLOT_BITMAP = ( 1 << sizeof(sample)/sizeof(short) ) - 1;

/**************************************************************************
*	
* This constant is used to specify how many word is required for 
* 1 sample data.
*	
**************************************************************************/	
	
	const int SLOT_SIZE = sizeof(sample)/sizeof(short);

/**************************************************************************
*	
* A plug-in class which provide context switching disable.
*	
**************************************************************************/	

	class criticalsection{
	public:
		virtual void enter()=0;		// disable context switch
		virtual void leave()=0;		// re-enable context switch
	};
	
/**************************************************************************
*	
* A plug-in class which setup and run SPORT. SPORT should used DMA which 
* employs double buffere algorithm.
*	
**************************************************************************/	

	class sport{
	protected:
		int port;
	public:
		sport( int aPort );
		virtual sample * curRxBuf( )=0;		// return currently available buffer to process
		virtual sample * curTxBuf( )=0;		// return currently available buffer to process
		virtual void initAndStart( int bufSize, sample * bufTxA, sample * bufRxA )=0;		// init sport and codec and then start transmission.
		virtual void stopAndWait()=0;		// stop to transmitt and then wait done
		virtual void clearInterrupt()=0;	// clear the interrupt at interrrupt handler. if needed.
	protected:
		virtual void error( int msg ) ;
	};

/**************************************************************************
*	
* A plug-in class which reset the AC97 codec by hardware. 
*	
**************************************************************************/	

	class reset{
	private:
		int mh;
	public:
		reset( int megaHertz );		// pass clock frequency.
		virtual void assert( int port )=0;
		virtual void deassert( int port )=0;
		virtual void wait1us( ); 	// dummy loop wait for 1 micro sec
	};
	
/**************************************************************************
*	
* Class which provide AD1819, AD188x, AD198x AC97 codec handling.
*	
**************************************************************************/	

	class framework{
	protected:
		short port, bufSize;
		short lastError;
		struct sample * bufTxA, * bufTxB, *bufRxA, *bufRxB;
		struct sample * curBufTx, *curBufRx;
	private:
		class criticalsection * criticalSection, * taskCriticalSection;
		class reset * reset;
		class sport * sport;
		short *cmdBuffer;
		short cmdPtr, bufferedCmdLen;
		short	state;
		virtual void iSendCommand( int adrs, int cmd );		
	public:
		framework( int aPort, int aBufSize );
		~framework();
		virtual void run();									// start to transfer with AC97
		virtual void stop();
		virtual void resetCodec();							// reset codec by hardware.
		virtual void installCriticalSection( 	criticalsection * cs,		// critical section to protect interrupt-time
												criticalsection * tcs = 0); // critical section to protect task-time
		virtual void installSport( class sport * sp );
		virtual void intstallReset( class reset * rst );
		virtual void handleInterrupt();
		virtual int currentState();							// return the internal state of framework
		virtual void tAC97Write( int adrs, int cmd );		// task-time 
		virtual void tAC97Read( int adrs );					// task-time
		virtual void iAC97Write( int adrs, int cmd );		// interrupt-time
		virtual void iAC97Read( int adrs );					// interrupt-time
	protected:					// have to be overriden by Application class
		virtual void error( int msg ) ;
	public:						// have to be overriden by Application class 
								// this method is actual hook for application.
								// this is called every DMA interrupt happen. 
		virtual void handleBuffer( struct sample * bufTx, 
									struct sample * bufRx )=0;
	};

};
#endif // INCL_MCM
